/**
 * 
 */
package org.ow2.opensuit.xml.base.html;
/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.opensuit.xml.base.enums.Quality;
import org.ow2.opensuit.xml.base.enums.ScaleMode;
import org.ow2.opensuit.xml.base.enums.ScriptAccess;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;

/**
 * This component allows to display a flash player,
 * several attributes are optional, but 
 * width/height/movie are required. Width and height
 * allow to set the size of the flash player frame.
 * Movie provides the Shockwave Flash file, that will
 * be play.
 * @author Adrien Ruffie / Pierre Smeyers
 * @author http://opensuit.ow2.org/
 * @version 1.0
 */
@XmlDoc("<p>This component allows to display a Flash Player, "
		+ "several attributes are optional, but "
		+ "width/height/movie are required. Width and height "
		+ "allow to set the size of the flash player frame. "
		+ "Movie provides the Shockwave Flash file, that will "
		+ "be play.<br/><br/>"
		+ "Nevertheless for security reasons OpenSUIT forbid to access internal resources. "
		+ "In order to access to your specified <b>.swf file</b> you must, use <b>Movie</b> attribute "
		+ "with this value: </p><br/><br/> 'http://{ip}:{port}/{context-root}/{OpenSUIT-url-servlet-mapping}/res/file.swf' "
		+ "<br/><br/><p>Flash Player try to load the specified <b>.swf file</b>, but with OpenSUIT security"
		+ " access you can't for example get <b>.swf file.</b></p><br/><br/> "
		+ "<br/><br/><br/>"
		+ "<b>WARNING:</b>"
		+ "<br/><br/>"
		+ "<p>To override OpenSUIT security access behavior you need to put in the same directory of "
		+ "your file <b>myfile.swf</b>, another file named <b>myfile.swf.res</b><br/> These two files "
		+ "must be available in your classpath.</p>"
		+ "<br/><br/><br/>"
		+ "<b>Example:</b>"
		+ "<br/><br/>"
		+ "'<'base:html.FlashPlayer Movie=\"http://localhost:8080/asolo-ui/ASolo/res/explorer.swf<br/>"
		+ "Height=\"350\"<br/>"
		+ "Width=\"350\"<br/>"
		+ "Menu=\"true\"<br/>"
		+ "/>"
		+ "<br/><br/>"
		+ "Your <b>explorer.swf</b> and these linked security access file <b>explorer.swf.res</b> must be "
		+ "located at:<br/> <b>WEB-INF/classes/explorer.swf</b> and <b>WEB-INF/classes/explorer.swf.res</b>")
@XmlElement
public class FlashPlayer implements IView {

	/*
	 * TODO Add the using of codebase, like in Applet Component
	 * instead affect all path in movie attributes:
	 * 
	 * now: Movie = http://localhost:8080/asolo-ui/ASolo/res/exporer.xml
	 * 
	 * future: Movie = explorer.xml
	 * 		   CodeBase = http://localhost:8080/asolo-ui/ASolo/res/
	 */
	@XmlDoc("Required attribute, represent the Shockwave Flash "+
			"file, that will be play in the player.")
	@XmlAttribute(name="Movie")
	private String movie;
	
	@XmlDoc("This optional attribute indicates whether "+
			"the player menu should be displayed.")
	@XmlAttribute(name="Menu", required=false)
	private boolean menu = false;
	
	@XmlDoc("This optional attribute indicates whether the "+
			"animation should be read, when it is loaded.")
	@XmlAttribute(name="Play", required=false)
	private boolean play = false;
	
	@XmlDoc("This optional attribute indicates whether the "+
			"animation is readed in loop, or it was stopped "+
			"after reading.")
	@XmlAttribute(name="Loop", required=false)
	private boolean loop = false;
	
	@XmlDoc("This required attribute indicates the width "+
			"of the flash player frame.")
	@XmlAttribute(name="Width")
	private int width;
	
	@XmlDoc("This required attribute indicates the height "+
			"of the flash player frame.")
	@XmlAttribute(name="Height")
	private int height;
	
	@XmlDoc("This optional attribute provide an alternative "+
			"text, if the flash player isn't displayed.")
	@XmlAttribute(name="Text", required=false)
	private String text = "Flash Player";
	
	@XmlDoc("This optional attribute allow to "+
			"set a transparent background.")
	@XmlAttribute(name="WMode", required=false)
	private boolean wMode = false;
	
	@XmlDoc("This optional attribute indicates whether Java "+
			"should be launched (required for use 'fscommand').")
	@XmlAttribute(name="SWLiveConnect", required=false)
	private boolean swLiveConnect = false;
	
	@XmlDoc("This optional attribute indicates the script "+
			"access behavior .")
	@XmlAttribute(name="AllowScriptAccess", required=false)
	private ScriptAccess allowScriptAccess = ScriptAccess.SAMEDOMAIN;
	
	@XmlDoc("This optional attribute indicates display quality.")
	@XmlAttribute(name="Quality", required=false)
	private Quality quality = Quality.MEDIUM;

	@XmlDoc("This optional attribute indicates display type.")
	@XmlAttribute(name="ScaleMode", required=false)
	private ScaleMode scaleMode = ScaleMode.SHOWALL;
	
	private final StringBuilder responseBuilder;
	
	public FlashPlayer(){
		this.responseBuilder = new StringBuilder();
	}
	
	/**
	 * This function is fire before the render process of FlashPlayer 
	 * View. It calculates the XHTML object tag result, which will be 
	 * display in render function.
	 * @param iRequest HttpServletRequest interface allow in this function,
	 * to access to request parameters, session, attributes, content type,
	 * encoding ... all informations for HTTP servlets
	 * @see org.ow2.opensuit.xml.interfaces.IRenderable#preRender(javax.servlet.http.HttpServletRequest)
	 * @throws Exception can throw generic exception
	 */
	public void preRender(final HttpServletRequest iRequest) throws Exception {
		
		// Add required parameters (with,heigh and movie)
		this.responseBuilder.append("<object type=\"application/x-shockwave-flash\" data=\"")
							.append(this.movie)
							.append("\" width=\"")
							.append(this.width)
							.append("\" height=\"")
							.append(this.height)
							.append("\">");
		
		/*
		 * Add required param "movie"
		 */
		this.responseBuilder.append("<param name=\"movie\" value=\"")
							.append(this.movie)
							.append("\"/>");
		
		/*
		 *  Check if other optional attributes are provided,
		 *  and add these to thank to <param> tag
		 */
		this.responseBuilder.append("<param name=\"menu\" value=\"")
							.append(this.menu)
							.append("\"/>");
		
		this.responseBuilder.append("<param name=\"play\" value=\"")
							.append(this.play)
							.append("\"/>");
		
		this.responseBuilder.append("<param name=\"loop\" value=\"")
							.append(this.loop)
							.append("\"/>");
		
		this.responseBuilder.append("<param name=\"wmode\" value=\"")
							.append(this.wMode)
							.append("\"/>");
		
		this.responseBuilder.append("<param name=\"swliveconnect\" value=\"")
							.append(this.swLiveConnect)
							.append("\"/>");
		
		this.responseBuilder.append("<param name=\"allowscriptaccess\" value=\"")
							.append(this.allowScriptAccess.getValue())
							.append("\"/>");
		
		this.responseBuilder.append("<param name=\"quality\" value=\"")
							.append(this.quality.getValue())
							.append("\"/>");
		
		this.responseBuilder.append("<param name=\"scalemode\" value=\"")
							.append(this.scaleMode.getValue())
							.append("\"/>");
		
		this.responseBuilder.append("<p>")
							.append(this.text)
							.append("</p>");
		
		/*
		 * TODO: flashvars 	 nom=valeur
		 * Transmettre une variable  lanimation.
		 * Pour en transmettre plusieurs,, utilisez
		 * un & intermerdiaire. Le & ne semble pas
		 * passer aux validateur du w3c, utilisez plutt &amp;. 
		 */
		
		this.responseBuilder.append("</object>");
	}

	/**
	 * This function just get the response writer, and write the flash
	 * player tag calculated just before in preRender method.
	 * @param iRequest HttpServletRequest interface allow in this function,
	 * to access to request parameters, session, attributes, content type,
	 * encoding ... all informations for HTTP servlets
	 * @param iResponse HttpServletResponse interface to provide HTTP-specific
	 * functionality in sending a response. Allow to access HTTP headers,
	 * cookies, status, encode redirected url, locale ..., and the writer
	 * in order to send the content of the response
	 * @see org.ow2.opensuit.xml.interfaces.IRenderable#render(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 * @throws Exception can throw generic exception
	 */
	public void render(final HttpServletRequest iRequest,
			final HttpServletResponse iResponse) throws Exception {
		final PrintWriter writer = iResponse.getWriter();
		writer.print(this.responseBuilder.toString());
	}

}
